@@ -31,7 +31,7 @@ module ActiveRecord # :nodoc:
 
       klass = instances.first.class
 
-      fresh_instances = klass.where(conditions).order(order)
+      fresh_instances = klass.unscoped.where(conditions).order(order)
       instances.each do |instance|
         matched_instance = fresh_instances.detect do |fresh_instance|
           keys.all? { |key| fresh_instance.send(key) == instance.send(key) }
